// Soubor GO2.CS
// Implementace tdy GrafickObjekt a jeho potomk
// Pklad klonovn:
// Tda GrafickObjekt implementuje rozhran ICloneable
// a metodu Clone definuje jako virtuln. 
// Metoda GrafickObjket.Clone() vol MemberwiseClone(), nebo
// zde - stejn jako ve td Bod a ZakroukovanBod - kopie po bitech
// naprosto vyhovuje. 
// Tdy seka a Krunice vak mus tuto metodu pekrt, 
// nebo potebuj hlubokou kopii sloek typu Bod.

using System;
using System.Collections;

// Vtov typ pro barvy
public enum barvy {erven, zelen, modr, bl}

//spolen pedek implementuje spolen vlastnosti vdch grafickch objekt
public class GrafickObjekt: ICloneable
{
	protected barvy barva;				
	protected bool nakreslen = false;
	public virtual object Clone()
	{
		return MemberwiseClone();
	}

	public virtual barvy Barva		// M vlastnost barva, pi zmn barvy se pekresl
	{
		set 
		{
			barva=value;
			if (nakreslen) Nakresli();
		}
		get
		{
			return barva;
		}
	}
	public GrafickObjekt(barvy _barva){Barva=_barva;}
	public GrafickObjekt(): this(barvy.bl) {}
	public virtual void Nakresli() // "Nakresl" obecn grafick objekt
	{
		nakreslen = true;
		Console.WriteLine("GrafickObjekt, barva "+Barva);
	}
}


// Bod zdd barvu 
// Metodu Nakresli nutno pedefinovat (bod se kresl jinak ne obecn grafick objekt)
// Aby bylo mono zachzet s Bodem jako s obecnm grafickm objektem, mus bt 
// metoda Nakresli virtuln
public class Bod: GrafickObjekt 
{
	private int x, y;						// Souadnice bodu
	public int X
	{
		set{ x = value;}
		get{ return x;} 
	}
	public int Y
	{
		set{ y = value; }
		get{ return y; }
	}
	public override void Nakresli()// "Kreslen" bodu
	{			
		nakreslen = true;
		Console.WriteLine("Kreslim Bod, barva " + Barva+ ", (" + X + ", "+ Y + 
			")");
	}
	public Bod(int _x, int _y, barvy _barva): base(_barva)
	{
		X = _x;
		Y = _y;
	}
}


// Zakroukovan bod zddil Barvu po Grafickm objektu (prostednictvm Bodu) 
// a souadnice od Bodu
public class ZakroukovanBod: Bod 
{
	public ZakroukovanBod(int _x, int _y, barvy _barva): base(_x, _y, _barva)
	{
	}
	private void NakresliKrouek()
	{
		Console.WriteLine("Kreslim krouzek");
	}
	public override void Nakresli() 
	{		// Pedefinovan metoda
		base.Nakresli();			// Vol metodu pedka
		NakresliKrouek();
	}
}


// seka je urena dvma krajnmi body
public class seka: GrafickObjekt 
{
	protected Bod a, b;					// Krajn body
	public Bod A
	{
		get	{ return a;}		
		set{ a = value;}
	}
	public Bod B
	{
		get{ return b;}
		set{ b = value;}
	}
	public override void Nakresli()// "Kreslen" seky
	{			
		Console.WriteLine("Kreslim Usecku, barva " +
			Barva + ", krajn body:");
		A.Nakresli();
		B.Nakresli();
	}
	public override object Clone()
	{
		seka vsledek = (seka)base.Clone();
        vsledek.A = (Bod)A.Clone();
		vsledek.B = (Bod)B.Clone();
		return vsledek;
	}

	public seka(int _x1, int _y1, // Konstruktor
		int _x2, int _y2, barvy _barva) 
		:base(_barva)				// Konstruktor pedka
	{
		A = new Bod(_x1, _y1, _barva);// Krajn body
		B = new Bod(_x2, _y2, _barva);
	}
}

// Kronice je urena stedem, co je Bod, a polomrem
public class Krunice: GrafickObjekt 
{
	protected Bod sted;
	protected int r;

	public override object Clone()
	{
		Krunice vsledek = (Krunice)base.Clone();
		vsledek.Sted = (Bod)Sted.Clone();
		return vsledek;
	}
	public int R
	{
		get{return r;}
		set{r = value;}
	}
	public Bod Sted
	{
		get{return sted;}
		set{sted = value;}
	}

	public override void Nakresli()
	{
		Console.WriteLine("Kreslm Krunici, barva " +
			Barva+", polomr " + R + ", sted: ");
		Sted.Nakresli();
	}

	public Krunice(int _x, int _y, int _r, barvy _barva) : base(_barva)
	{
		Sted = new Bod(_x, _y, _barva);
		R = _r;
	}

}
// Njak jednoduch test polymorfnho chovn
class Program
{
	public static void Main()
	{
		// Vytvome dv instance, apk jejich klony
		// a v klonech nco zmnme.
		// Vsledek vypeme - ukazujue, e vznikla hlubok kopie
		// Kdybychom nepekryli metodu Clone ve tdch Krunice a seka,
		// vznikla by mlk kopie, tj. nap. zmna krajnho bodu klonu seky 
		// by zpsobila  zmnu krajnho bodu pvodn instance.
		// Vyzkouejte si to - zakomentujte metodu Clone ve tdch Krunice
		// a seka a spuste tento program

		seka u = new seka(1,2,3,4,barvy.modr);
		Krunice k = new Krunice(8, 9, 10, barvy.zelen); 
		seka uu = (seka)u.Clone();
		uu.A.X=12345;
		u.Nakresli();
		uu.Nakresli();
		Krunice kk = (Krunice)k.Clone();
		kk.R = 88;
		kk.Nakresli();
		k.Nakresli();
	}
}