// Soubor SERIAL.CS
// Implementace tdy GrafickObjekt a jeho potomk
// Navc tda Obrzek, pedstavujc kolekci grafickch objekt.
// Tato tda m metody Pidej(), je pid grafick objekt do kontehjneru,
// nakresli, je nakresl vechny objekty v kontejneru - tedy cel obrzek,
// Ulo(), je ulo cel obrzek do souboru pomoc serializace, a 
// Nati() , je nate obrzek ze souboru.


using System;
using System.IO;
using System.Collections;
//using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;

// Vtov typ pro barvy
public enum barvy {erven, zelen, modr, bl}

//spolen pedek implementuje spolen vlastnosti vdch grafickch objekt
[Serializable]
public class GrafickObjekt			
{
	protected barvy barva;				
	protected bool nakreslen = false;

	public virtual barvy Barva		// M vlastnost barva, pi zmn barvy se pekresl
	{
		set 
		{
			barva=value;
			if (nakreslen) Nakresli();
		}
		get
		{
			return barva;
		}
	}
	public GrafickObjekt(barvy _barva){Barva=_barva;}
	public GrafickObjekt(): this(barvy.bl) {}
	public virtual void Nakresli() // "Nakresl" obecn grafick objekt
	{
		nakreslen = true;
		Console.WriteLine("GrafickObjekt, barva "+Barva);
	}
}


// Bod zdd barvu 
// Metodu Nakresli nutno pedefinovat (bod se kresl jinak ne obecn grafick objekt)
// Aby bylo mono zachzet s Bodem jako s obecnm grafickm objektem, mus bt 
// metoda Nakresli virtuln
[Serializable]
public class Bod: GrafickObjekt 
{
	private int x, y;						// Souadnice bodu
	public int X
	{
		set{ x = value;}
		get{ return x;} 
	}
	public int Y
	{
		set{ y = value; }
		get{ return y; }
	}

	public override void Nakresli()// "Kreslen" bodu
	{			
		nakreslen = true;
		Console.WriteLine("Kreslm Bod, barva " + Barva+ ", (" + X + ", "+ Y + 
			")");
	}
	public Bod(int _x, int _y, barvy _barva): base(_barva)
	{
		X = _x;
		Y = _y;
	}
}


// Zakroukovan bod zddil Barvu po Grafickm objektu (prostednictvm Bodu) 
// a souadnice od Bodu
[Serializable]
public class ZakroukovanBod: Bod 
{
	public ZakroukovanBod(int _x, int _y, barvy _barva): base(_x, _y, _barva)
	{
	}
	private void NakresliKrouek()
	{
		Console.WriteLine("Kreslm krouek");
	}
	public override void Nakresli() 
	{		// Pedefinovan metoda
		base.Nakresli();			// Vol metodu pedka
		NakresliKrouek();
	}
}


// seka je urena dvma krajnmi body
[Serializable]
public class seka: GrafickObjekt 
{
	protected Bod a, b;					// Krajn body
	public Bod A
	{
		get	{ return a;}		
		set{ a = value;}
	}
	public Bod B
	{
		get{ return b;}
		set{ b = value;}
	}
	public override void Nakresli()// "Kreslen" seky
	{			
		Console.WriteLine("Kreslm seku, barva " +
			Barva + ", krajn body:");
		A.Nakresli();
		B.Nakresli();
	}
	public seka(int _x1, int _y1, // Konstruktor
		int _x2, int _y2, barvy _barva) 
		:base(_barva)				// Konstruktor pedka
	{
		A = new Bod(_x1, _y1, _barva);// Krajn body
		B = new Bod(_x2, _y2, _barva);
	}
}

// Krunice je urena stedem, co je Bod, a polomrem
[Serializable]
public class Krunice: GrafickObjekt 
{
	protected Bod sted;
	protected int r;

	public int R
	{
		get{return r;}
		set{r = value;}
	}
	public Bod Sted
	{
		get{return sted;}
		set{sted = value;}
	}

	public override void Nakresli()
	{
		Console.WriteLine("Kreslm Krunici, barva " +
			Barva+", polomr " + R + ", sted: ");
		Sted.Nakresli();
	}

	public Krunice(int _x, int _y, int _r, barvy _barva) : base(_barva)
	{
		Sted = new Bod(_x, _y, _barva);
		R = _r;
	}

}

[Serializable]
public class Obrzek
{
	ArrayList obr = new ArrayList();
	public void Pidej(GrafickObjekt go)
	{
		obr.Add(go);
	}
	public void Nakresli()
	{
		foreach(GrafickObjekt go in obr) go.Nakresli();
	}

	public void Ulo(string soubor)
	{
		FileStream fs = File.Create(soubor);
		BinaryFormatter bf = new BinaryFormatter();
		bf.Serialize(fs, obr);
		fs.Close();
	}
	public void Nati(string soubor)
	{
		FileStream fs = File.Open(soubor, FileMode.Open);
		BinaryFormatter bf = new BinaryFormatter();
		obr = (ArrayList)bf.Deserialize(fs);
		fs.Close();
	}

}

// Njak jednoduch test polymorfnho chovn
class Program
{
	static Obrzek Kresba = new Obrzek();
	public static void VytvoObrzek()
	{
		Kresba.Pidej(new Bod(5,6,barvy.bl));
		Kresba.Pidej(new seka(1,2,3,4,barvy.modr));
		Kresba.Pidej(new Krunice(8, 9, 10, barvy.zelen));
		Kresba.Pidej(new ZakroukovanBod(6,9,barvy.erven));
	}

	public static void Main()
	{
		string jmno = "obrazek.obr";
		VytvoObrzek();
		Console.WriteLine("Vytvoen obrzek:");
		Kresba.Nakresli();
		Kresba.Ulo(jmno);
		Kresba = new Obrzek();
		Kresba.Nati(jmno);
		Console.WriteLine("Naten obrzek:");
		Kresba.Nakresli();

	}
}